@@ -1,6 +1,6 @@
 // Scintilla source code edit control
 // PlatGTK.cxx - implementation of platform facilities on GTK+/Linux
-// Copyright 1998-2003 by Neil Hodgson <neilh@scintilla.org>
+// Copyright 1998-2004 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
 #include <string.h>
@@ -37,16 +37,7 @@
 
 #if GTK_MAJOR_VERSION >= 2
 #define USE_PANGO 1
-#include <iconv.h>
-const iconv_t iconvhBad = (iconv_t)(-1);
-// Since various versions of iconv can not agree on whether the src argument
-// is char ** or const char ** provide a templatised adaptor.
-template<typename T>
-size_t iconv_adaptor(size_t(*f_iconv)(iconv_t, T, size_t *, char **, size_t *),
-		iconv_t cd, char** src, size_t *srcleft,
-		char **dst, size_t *dstleft) {
-	return f_iconv(cd, (T)src, srcleft, dst, dstleft);
-}
+#include "Converter.h"
 #endif
 
 #ifdef _MSC_VER
@@ -201,10 +192,14 @@ Palette::Palette() {
 	allowRealization = false;
 	allocatedPalette = 0;
 	allocatedLen = 0;
+	size = 100;
+	entries = new ColourPair[size];
 }
 
 Palette::~Palette() {
 	Release();
+	delete []entries;
+	entries = 0;
 }
 
 void Palette::Release() {
@@ -212,6 +207,9 @@ void Palette::Release() {
 	delete [](reinterpret_cast<GdkColor *>(allocatedPalette));
 	allocatedPalette = 0;
 	allocatedLen = 0;
+	delete []entries;
+	size = 100;
+	entries = new ColourPair[size];
 }
 
 // This method either adds a colour to the list of wanted colours (want==true)
@@ -219,18 +217,27 @@ void Palette::Release() {
 // This is one method to make it easier to keep the code for wanting and retrieving in sync.
 void Palette::WantFind(ColourPair &cp, bool want) {
 	if (want) {
-		for (int i = 0; i < used; i++) {
+		for (int i=0; i < used; i++) {
 			if (entries[i].desired == cp.desired)
 				return;
 		}
 
-		if (used < numEntries) {
-			entries[used].desired = cp.desired;
-			entries[used].allocated.Set(cp.desired.AsLong());
-			used++;
+		if (used >= size) {
+			int sizeNew = size * 2;
+			ColourPair *entriesNew = new ColourPair[sizeNew];
+			for (int j=0; j<size; j++) {
+				entriesNew[j] = entries[j];
+			}
+			delete []entries;
+			entries = entriesNew;
+			size = sizeNew;
 		}
+
+		entries[used].desired = cp.desired;
+		entries[used].allocated.Set(cp.desired.AsLong());
+		used++;
 	} else {
-		for (int i = 0; i < used; i++) {
+		for (int i=0; i < used; i++) {
 			if (entries[i].desired == cp.desired) {
 				cp.allocated = entries[i].allocated;
 				return;
@@ -295,6 +302,8 @@ static const char *CharacterSetName(int characterSet) {
 		return "*-*";
 	case SC_CHARSET_RUSSIAN:
 		return "*-r";
+	case SC_CHARSET_CYRILLIC:
+		return "*-cp1251";
 	case SC_CHARSET_SHIFTJIS:
 		return "jisx0208.1983-*";
 	case SC_CHARSET_SYMBOL:
@@ -311,6 +320,8 @@ static const char *CharacterSetName(int characterSet) {
 		return "*-*";
 	case SC_CHARSET_THAI:
 		return "*-1";
+	case SC_CHARSET_8859_15:
+		return "iso8859-15";
 	default:
 		return "*-*";
 	}
@@ -654,7 +665,8 @@ Font::Font() : id(0) {}
 
 Font::~Font() {}
 
-void Font::Create(const char *faceName, int characterSet, int size, bool bold, bool italic) {
+void Font::Create(const char *faceName, int characterSet, int size,
+	bool bold, bool italic, bool) {
 	Release();
 	id = FontCached::FindOrCreate(faceName, characterSet, size, bold, italic);
 }
@@ -677,9 +689,9 @@ class SurfaceImpl : public Surface {
 #ifdef USE_PANGO
 	PangoContext *pcontext;
 	PangoLayout *layout;
-	iconv_t iconvh;
+	Converter conv;
 	int characterSet;
-	void SetIconv(int characterSet_);
+	void SetConverter(int characterSet_);
 #endif
 public:
 	SurfaceImpl();
@@ -701,6 +713,8 @@ public:
 	void FillRectangle(PRectangle rc, ColourAllocated back);
 	void FillRectangle(PRectangle rc, Surface &surfacePattern);
 	void RoundedRectangle(PRectangle rc, ColourAllocated fore, ColourAllocated back);
+	void AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill, int alphaFill,
+		ColourAllocated outline, int alphaOutline, int flags);
 	void Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back);
 	void Copy(PRectangle rc, Point from, Surface &surfaceSource);
 
@@ -750,6 +764,8 @@ const char *CharacterSetID(int characterSet) {
 		return "ASCII";
 	case SC_CHARSET_RUSSIAN:
 		return "KOI8-R";
+	case SC_CHARSET_CYRILLIC:
+		return "CP1251";
 	case SC_CHARSET_SHIFTJIS:
 		return "SHIFT-JIS";
 	case SC_CHARSET_SYMBOL:
@@ -766,6 +782,8 @@ const char *CharacterSetID(int characterSet) {
 		return "";
 	case SC_CHARSET_THAI:
 		return "ISO-8859-1";
+	case SC_CHARSET_8859_15:
+		return "ISO-8859-15";
 	default:
 		return "";
 	}
@@ -773,16 +791,10 @@ const char *CharacterSetID(int characterSet) {
 
 #ifdef USE_PANGO
 
-void SurfaceImpl::SetIconv(int characterSet_) {
+void SurfaceImpl::SetConverter(int characterSet_) {
 	if (characterSet != characterSet_) {
-		if (iconvh != iconvhBad)
-			iconv_close(iconvh);
-		iconvh = iconvhBad;
 		characterSet = characterSet_;
-		const char *source = CharacterSetID(characterSet);
-		if (*source) {
-			iconvh = iconv_open("UTF-8", source);
-		}
+		conv.Open("UTF-8", CharacterSetID(characterSet));
 	}
 }
 #endif
@@ -790,7 +802,7 @@ void SurfaceImpl::SetIconv(int characterSet_) {
 SurfaceImpl::SurfaceImpl() : et(singleByte), drawable(0), gc(0), ppixmap(0),
 x(0), y(0), inited(false), createdGC(false)
 #ifdef USE_PANGO
-, pcontext(0), layout(0), iconvh(iconvhBad), characterSet(-1)
+, pcontext(0), layout(0), characterSet(-1)
 #endif
 {
 }
@@ -817,9 +829,7 @@ void SurfaceImpl::Release() {
 	if (pcontext)
 		g_object_unref(pcontext);
 	pcontext = 0;
-	if (iconvh != iconvhBad)
-		iconv_close(iconvh);
-	iconvh = iconvhBad;
+	conv.Close();
 	characterSet = -1;
 #endif
 	x = 0;
@@ -862,6 +872,8 @@ void SurfaceImpl::Init(SurfaceID sid, WindowID WID_NAME) {
 #endif
 	drawable = drawable_;
 	gc = gdk_gc_new(drawable_);
+	// Ask for lines that do not paint the last pixel so is like Win32
+	gdk_gc_set_line_attributes(gc, 0, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
 	createdGC = true;
 	inited = true;
 }
@@ -882,6 +894,8 @@ void SurfaceImpl::InitPixMap(int width, int height, Surface *surface_, WindowID
 		ppixmap = gdk_pixmap_new(surfImpl->drawable, width, height, -1);
 	drawable = ppixmap;
 	gc = gdk_gc_new(surfImpl->drawable);
+	// Ask for lines that do not paint the last pixel so is like Win32
+	gdk_gc_set_line_attributes(gc, 0, GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
 	createdGC = true;
 	inited = true;
 }
@@ -1002,6 +1016,88 @@ void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourAllocated fore, ColourAl
 	}
 }
 
+// Plot a point into a guint32 buffer symetrically to all 4 qudrants
+static void AllFour(guint32 *pixels, int stride, int width, int height, int x, int y, guint32 val) {
+	pixels[y*stride+x] = val;
+	pixels[y*stride+width-1-x] = val;
+	pixels[(height-1-y)*stride+x] = val;
+	pixels[(height-1-y)*stride+width-1-x] = val;
+}
+
+static unsigned int GetRValue(unsigned int co) {
+	return (co >> 16) & 0xff;
+}
+
+static unsigned int GetGValue(unsigned int co) {
+	return (co >> 8) & 0xff;
+}
+
+static unsigned int GetBValue(unsigned int co) {
+	return co & 0xff;
+}
+
+#if GTK_MAJOR_VERSION < 2
+void SurfaceImpl::AlphaRectangle(PRectangle rc, int , ColourAllocated , int , ColourAllocated outline, int , int ) {
+	if (gc && drawable) {
+		// Can't use GdkPixbuf on GTK+ 1.x, so draw an outline rather than use alpha.
+		PenColour(outline);
+		gdk_draw_rectangle(drawable, gc, 0,
+		                   rc.left, rc.top,
+		                   rc.right - rc.left - 1, rc.bottom - rc.top - 1);
+	}
+}
+#else
+void SurfaceImpl::AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill, int alphaFill,
+		ColourAllocated outline, int alphaOutline, int flags) {
+	if (gc && drawable) {
+		int width = rc.Width();
+		int height = rc.Height();
+		// Ensure not distorted too much by corners when small
+		cornerSize = Platform::Minimum(cornerSize, (Platform::Minimum(width, height) / 2) - 2);
+		// Make a 32 bit deep pixbuf with alpha
+		GdkPixbuf *pixalpha = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, width, height);
+
+		guint8 pixVal[4] = {0};
+		guint32 valEmpty = *(reinterpret_cast<guint32 *>(pixVal));
+		pixVal[0] = GetRValue(fill.AsLong());
+		pixVal[1] = GetGValue(fill.AsLong());
+		pixVal[2] = GetBValue(fill.AsLong());
+		pixVal[3] = alphaFill;
+		guint32 valFill = *(reinterpret_cast<guint32 *>(pixVal));
+		pixVal[0] = GetRValue(outline.AsLong());
+		pixVal[1] = GetGValue(outline.AsLong());
+		pixVal[2] = GetBValue(outline.AsLong());
+		pixVal[3] = alphaOutline;
+		guint32 valOutline = *(reinterpret_cast<guint32 *>(pixVal));
+		guint32 *pixels = reinterpret_cast<guint32 *>(gdk_pixbuf_get_pixels(pixalpha));
+		int stride = gdk_pixbuf_get_rowstride(pixalpha) / 4;
+		for (int y=0; y<height; y++) {
+			for (int x=0; x<width; x++) {
+				if ((x==0) || (x==width-1) || (y == 0) || (y == height-1)) {
+					pixels[y*stride+x] = valOutline;
+				} else {
+					pixels[y*stride+x] = valFill;
+				}
+			}
+		}
+		for (int c=0;c<cornerSize; c++) {
+			for (int x=0;x<c+1; x++) {
+				AllFour(pixels, stride, width, height, x, c-x, valEmpty);
+			}
+		}
+		for (int x=1;x<cornerSize; x++) {
+			AllFour(pixels, stride, width, height, x, cornerSize-x, valOutline);
+		}
+
+		// Draw with alpha
+		gdk_draw_pixbuf(drawable, gc, pixalpha,
+			0,0, rc.left,rc.top, width,height, GDK_RGB_DITHER_NORMAL, 0, 0);
+
+		g_object_unref(pixalpha);
+	}
+}
+#endif
+
 void SurfaceImpl::Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back) {
 	PenColour(back);
 	gdk_draw_arc(drawable, gc, 1,
@@ -1038,40 +1134,7 @@ static size_t UTF8Len(char ch) {
 		return 3;
 }
 
-#ifdef USE_PANGO
-static char *UTF8FromIconv(iconv_t iconvh, const char *s, int len) {
-	if (iconvh != ((iconv_t)(-1))) {
-		char *utfForm = new char[len*3+1];
-		char *pin = const_cast<char *>(s);
-		size_t inLeft = len;
-		char *pout = utfForm;
-		size_t outLeft = len*3+1;
-		size_t conversions = iconv_adaptor(iconv, iconvh, &pin, &inLeft, &pout, &outLeft);
-		if (conversions != ((size_t)(-1))) {
-			*pout = '\0';
-			return utfForm;
-		}
-		delete []utfForm;
-	}
-	return 0;
-}
-
-static size_t MultiByteLenFromIconv(iconv_t iconvh, const char *s, size_t len) {
-	for (size_t lenMB=1; (lenMB<4) && (lenMB <= len); lenMB++) {
-		char wcForm[2];
-		char *pin = const_cast<char *>(s);
-		size_t inLeft = lenMB;
-		char *pout = wcForm;
-		size_t outLeft = 2;
-		size_t conversions = iconv_adaptor(iconv, iconvh, &pin, &inLeft, &pout, &outLeft);
-		if (conversions != ((size_t)(-1))) {
-			return lenMB;
-		}
-	}
-	return 1;
-}
-
-static char *UTF8FromLatin1(const char *s, int len) {
+char *UTF8FromLatin1(const char *s, int &len) {
 	char *utfForm = new char[len*2+1];
 	size_t lenU = 0;
 	for (int i=0;i<len;i++) {
@@ -1084,9 +1147,46 @@ static char *UTF8FromLatin1(const char *s, int len) {
 		}
 	}
 	utfForm[lenU] = '\0';
+	len = lenU;
 	return utfForm;
 }
 
+#ifdef USE_PANGO
+static char *UTF8FromIconv(const Converter &conv, const char *s, int &len) {
+	if (conv) {
+		char *utfForm = new char[len*3+1];
+		char *pin = const_cast<char *>(s);
+		size_t inLeft = len;
+		char *pout = utfForm;
+		size_t outLeft = len*3+1;
+		size_t conversions = conv.Convert(&pin, &inLeft, &pout, &outLeft);
+		if (conversions != ((size_t)(-1))) {
+			*pout = '\0';
+			len = pout - utfForm;
+			return utfForm;
+		}
+		delete []utfForm;
+	}
+	return 0;
+}
+
+// Work out how many bytes are in a character by trying to convert using iconv,
+// returning the first length that succeeds.
+static size_t MultiByteLenFromIconv(const Converter &conv, const char *s, size_t len) {
+	for (size_t lenMB=1; (lenMB<4) && (lenMB <= len); lenMB++) {
+		char wcForm[2];
+		char *pin = const_cast<char *>(s);
+		size_t inLeft = lenMB;
+		char *pout = wcForm;
+		size_t outLeft = 2;
+		size_t conversions = conv.Convert(&pin, &inLeft, &pout, &outLeft);
+		if (conversions != ((size_t)(-1))) {
+			return lenMB;
+		}
+	}
+	return 1;
+}
+
 static char *UTF8FromGdkWChar(GdkWChar *wctext, int wclen) {
 	char *utfForm = new char[wclen*3+1];	// Maximum of 3 UTF-8 bytes per character
 	size_t lenU = 0;
@@ -1107,7 +1207,7 @@ static char *UTF8FromGdkWChar(GdkWChar *wctext, int wclen) {
 	return utfForm;
 }
 
-static char *UTF8FromDBCS(const char *s, int len) {
+static char *UTF8FromDBCS(const char *s, int &len) {
 	GdkWChar *wctext = new GdkWChar[len + 1];
 	GdkWChar *wcp = wctext;
 	int wclen = gdk_mbstowcs(wcp, s, len);
@@ -1120,6 +1220,7 @@ static char *UTF8FromDBCS(const char *s, int len) {
 
 	char *utfForm = UTF8FromGdkWChar(wctext, wclen);
 	delete []wctext;
+	len = strlen(utfForm);
 	return utfForm;
 }
 
@@ -1154,8 +1255,8 @@ void SurfaceImpl::DrawTextBase(PRectangle rc, Font &font_, int ybase, const char
 				pango_layout_set_text(layout, s, len);
 			} else {
 				if (!utfForm) {
-					SetIconv(PFont(font_)->characterSet);
-					utfForm = UTF8FromIconv(iconvh, s, len);
+					SetConverter(PFont(font_)->characterSet);
+					utfForm = UTF8FromIconv(conv, s, len);
 				}
 				if (!utfForm) {	// iconv failed so try DBCS if DBCS mode
 					if (et == dbcs) {
@@ -1166,7 +1267,7 @@ void SurfaceImpl::DrawTextBase(PRectangle rc, Font &font_, int ybase, const char
 				if (!utfForm) {	// iconv and DBCS failed so treat as Latin1
 					utfForm = UTF8FromLatin1(s, len);
 				}
-				pango_layout_set_text(layout, utfForm, strlen(utfForm));
+				pango_layout_set_text(layout, utfForm, len);
 			}
 			pango_layout_set_font_description(layout, PFont(font_)->pfd);
 			PangoLayoutLine *pll = pango_layout_get_line(layout,0);
@@ -1184,13 +1285,18 @@ void SurfaceImpl::DrawTextBase(PRectangle rc, Font &font_, int ybase, const char
 		bool draw8bit = true;
 		if (et != singleByte) {
 			GdkWChar wctext[maxLengthTextRun];
+			if (len >= maxLengthTextRun)
+				len = maxLengthTextRun-1;
 			int wclen;
 			if (et == UTF8) {
 				wclen = UCS2FromUTF8(s, len,
-					reinterpret_cast<wchar_t *>(wctext), maxLengthTextRun - 1);
+					static_cast<wchar_t *>(static_cast<void *>(wctext)), maxLengthTextRun - 1);
 			} else {	// dbcs, so convert using current locale
+				char sMeasure[maxLengthTextRun];
+				memcpy(sMeasure, s, len);
+				sMeasure[len] = '\0';
 				wclen = gdk_mbstowcs(
-					wctext, s, maxLengthTextRun - 1);
+					wctext, sMeasure, maxLengthTextRun - 1);
 			}
 			if (wclen > 0) {
 				draw8bit = false;
@@ -1201,11 +1307,11 @@ void SurfaceImpl::DrawTextBase(PRectangle rc, Font &font_, int ybase, const char
 					gdk_draw_text_wc(drawable, PFont(font_)->pfont, gc,
 							 x, ybase, wcp, lenDraw);
 					wclen -= lenDraw;
-					wcp += lenDraw;
 					if (wclen > 0) {	// Avoid next calculation if possible as may be expensive
 						x += gdk_text_width_wc(PFont(font_)->pfont,
 								       wcp, lenDraw);
 					}
+					wcp += lenDraw;
 				}
 			}
 		}
@@ -1215,10 +1321,10 @@ void SurfaceImpl::DrawTextBase(PRectangle rc, Font &font_, int ybase, const char
 				gdk_draw_text(drawable, PFont(font_)->pfont, gc,
 				              x, ybase, s, lenDraw);
 				len -= lenDraw;
-				s += lenDraw;
 				if (len > 0) {	// Avoid next calculation if possible as may be expensive
 					x += gdk_text_width(PFont(font_)->pfont, s, lenDraw);
 				}
+				s += lenDraw;
 			}
 		}
 	}
@@ -1266,43 +1372,42 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, int *positi
 			if (et == UTF8) {
 				// Simple and direct as UTF-8 is native Pango encoding
 				pango_layout_set_text(layout, s, len);
-				PangoLayoutIter *iter = pango_layout_get_iter (layout);
+				PangoLayoutIter *iter = pango_layout_get_iter(layout);
 				pango_layout_iter_get_cluster_extents(iter, NULL, &pos);
 				int i = 0;
-				while (pango_layout_iter_next_cluster (iter)) {
+				while (pango_layout_iter_next_cluster(iter)) {
 					pango_layout_iter_get_cluster_extents(iter, NULL, &pos);
 					int position = PANGO_PIXELS(pos.x);
-					int curIndex = pango_layout_iter_get_index (iter);
+					int curIndex = pango_layout_iter_get_index(iter);
 					while (i < curIndex) {
 						positions[i++] = position;
 					}
 				}
 				while (i < lenPositions)
 					positions[i++] = PANGO_PIXELS(pos.x + pos.width);
-				pango_layout_iter_free (iter);
+				pango_layout_iter_free(iter);
 				PLATFORM_ASSERT(i == lenPositions);
 			} else {
 				int positionsCalculated = 0;
 				if (et == dbcs) {
-					SetIconv(PFont(font_)->characterSet);
-					char *utfForm = UTF8FromIconv(iconvh, s, len);
+					SetConverter(PFont(font_)->characterSet);
+					char *utfForm = UTF8FromIconv(conv, s, len);
 					if (utfForm) {
 						// Convert to UTF-8 so can ask Pango for widths, then
 						// Loop through UTF-8 and DBCS forms, taking account of different
 						// character byte lengths.
-						iconv_t iconvhMeasure =
-							iconv_open("UCS-2", CharacterSetID(characterSet));
+						Converter convMeasure("UCS-2", CharacterSetID(characterSet));
 						pango_layout_set_text(layout, utfForm, strlen(utfForm));
 						int i = 0;
 						int utfIndex = 0;
-						PangoLayoutIter *iter = pango_layout_get_iter (layout);
+						PangoLayoutIter *iter = pango_layout_get_iter(layout);
 						pango_layout_iter_get_cluster_extents(iter, NULL, &pos);
-						while (pango_layout_iter_next_cluster (iter)) {
+						while (pango_layout_iter_next_cluster(iter)) {
 							pango_layout_iter_get_cluster_extents (iter, NULL, &pos);
 							int position = PANGO_PIXELS(pos.x);
-							int utfIndexNext = pango_layout_iter_get_index (iter);
+							int utfIndexNext = pango_layout_iter_get_index(iter);
 							while (utfIndex < utfIndexNext) {
-								size_t lenChar = MultiByteLenFromIconv(iconvhMeasure, s+i, len-i);
+								size_t lenChar = MultiByteLenFromIconv(convMeasure, s+i, len-i);
 								//size_t lenChar = mblen(s+i, MB_CUR_MAX);
 								while (lenChar--) {
 									positions[i++] = position;
@@ -1313,25 +1418,24 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, int *positi
 						}
 						while (i < lenPositions)
 							positions[i++] = PANGO_PIXELS(pos.x + pos.width);
-						pango_layout_iter_free (iter);
+						pango_layout_iter_free(iter);
 						delete []utfForm;
 						PLATFORM_ASSERT(i == lenPositions);
-						iconv_close(iconvhMeasure);
 					}
 				}
 				if (positionsCalculated < 1 ) {
 					// Either Latin1 or DBCS conversion failed so treat as Latin1.
 					bool useGFree = false;
-					SetIconv(PFont(font_)->characterSet);
-					char *utfForm = UTF8FromIconv(iconvh, s, len);
+					SetConverter(PFont(font_)->characterSet);
+					char *utfForm = UTF8FromIconv(conv, s, len);
 					if (!utfForm) {
 						utfForm = UTF8FromLatin1(s, len);
 					}
-					pango_layout_set_text(layout, utfForm, strlen(utfForm));
+					pango_layout_set_text(layout, utfForm, len);
 					int i = 0;
-					PangoLayoutIter *iter = pango_layout_get_iter (layout);
+					PangoLayoutIter *iter = pango_layout_get_iter(layout);
 					pango_layout_iter_get_cluster_extents(iter, NULL, &pos);
-					while (pango_layout_iter_next_cluster (iter)) {
+					while (pango_layout_iter_next_cluster(iter)) {
 						pango_layout_iter_get_cluster_extents(iter, NULL, &pos);
 						positions[i++] = PANGO_PIXELS(pos.x);
 					}
@@ -1356,13 +1460,18 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, int *positi
 		bool measure8bit = true;
 		if (et != singleByte) {
 			GdkWChar wctext[maxLengthTextRun];
+			if (len >= maxLengthTextRun)
+				len = maxLengthTextRun-1;
 			int wclen;
 			if (et == UTF8) {
 				wclen = UCS2FromUTF8(s, len,
-					reinterpret_cast<wchar_t *>(wctext), maxLengthTextRun - 1);
+					static_cast<wchar_t *>(static_cast<void *>(wctext)), maxLengthTextRun - 1);
 			} else {	// dbcsMode, so convert using current locale
+				char sDraw[maxLengthTextRun];
+				memcpy(sDraw, s, len);
+				sDraw[len] = '\0';
 				wclen = gdk_mbstowcs(
-					wctext, s, maxLengthTextRun - 1);
+					wctext, sDraw, maxLengthTextRun - 1);
 			}
 			if (wclen > 0) {
 				measure8bit = false;
@@ -1372,11 +1481,13 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, int *positi
 				for (int iU = 0; iU < wclen; iU++) {
 					int width = gdk_char_width_wc(gf, wctext[iU]);
 					totalWidth += width;
-					size_t lenChar;
+					int lenChar;
 					if (et == UTF8) {
 						lenChar = UTF8Len(s[i]);
 					} else {
 						lenChar = mblen(s+i, MB_CUR_MAX);
+						if (lenChar < 0)
+							lenChar = 1;
 					}
 					while (lenChar--) {
 						positions[i++] = totalWidth;
@@ -1419,14 +1530,13 @@ int SurfaceImpl::WidthText(Font &font_, const char *s, int len) {
 					utfForm = UTF8FromDBCS(s, len);
 				}
 				if (!utfForm) {	// DBCS failed so treat as iconv
-					SetIconv(PFont(font_)->characterSet);
-					utfForm = UTF8FromIconv(iconvh, s, len);
+					SetConverter(PFont(font_)->characterSet);
+					utfForm = UTF8FromIconv(conv, s, len);
 				}
 				if (!utfForm) {	// g_locale_to_utf8 failed so treat as Latin1
 					utfForm = UTF8FromLatin1(s, len);
 				}
-				pango_layout_set_text(layout, utfForm, strlen(utfForm));
-				len = strlen(utfForm);
+				pango_layout_set_text(layout, utfForm, len);
 			}
 			PangoLayoutLine *pangoLine = pango_layout_get_line(layout, 0);
 			pango_layout_line_get_extents(pangoLine, NULL, &pos);
@@ -1440,7 +1550,8 @@ int SurfaceImpl::WidthText(Font &font_, const char *s, int len) {
 #endif
 		if (et == UTF8) {
 			GdkWChar wctext[maxLengthTextRun];
-			size_t wclen = UCS2FromUTF8(s, len, (wchar_t *)wctext, sizeof(wctext) / sizeof(GdkWChar) - 1);
+			size_t wclen = UCS2FromUTF8(s, len, static_cast<wchar_t *>(static_cast<void *>(wctext)),
+				sizeof(wctext) / sizeof(GdkWChar) - 1);
 			wctext[wclen] = L'\0';
 			return gdk_text_width_wc(PFont(font_)->pfont, wctext, wclen);
 		} else {
@@ -1474,10 +1585,12 @@ int SurfaceImpl::WidthChar(Font &font_, char ch) {
 // rarely used in code.
 
 // This string contains a good range of characters to test for size.
-const char largeSizeString[] = " `~!@#$%^&*()-_=+\\|[]{};:\"\'<,>.?/1234567890"
-                               "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+//const char largeSizeString[] = " `~!@#$%^&*()-_=+\\|[]{};:\"\'<,>.?/1234567890"
+//                               "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+#ifndef FAST_WAY
 const char sizeString[] = "`~!@#$%^&*()-_=+\\|[]{};:\"\'<,>.?/1234567890"
                           "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+#endif
 
 int SurfaceImpl::Ascent(Font &font_) {
 	if (!(font_.GetID()))
@@ -1495,9 +1608,12 @@ int SurfaceImpl::Ascent(Font &font_) {
 		ascent = PFont(font_)->ascent;
 	}
 #endif
-	if (ascent == 0) {
+	if ((ascent == 0) && (PFont(font_)->pfont)) {
 		ascent = PFont(font_)->pfont->ascent;
 	}
+	if (ascent == 0) {
+		ascent = 1;
+	}
 	FontMutexUnlock();
 	return ascent;
 #else
@@ -1640,20 +1756,20 @@ void Window::SetPositionRelative(PRectangle rc, Window relativeTo) {
 		oy = 0;
 
 	/* do some corrections to fit into screen */
-	int sizey = rc.bottom - rc.top;
 	int sizex = rc.right - rc.left;
+	int sizey = rc.bottom - rc.top;
 	int screenWidth = gdk_screen_width();
 	int screenHeight = gdk_screen_height();
 	if (sizex > screenWidth)
 		ox = 0; /* the best we can do */
-	else
-		if (ox + sizex > screenWidth)
-			ox -= ox + sizex - screenWidth;
+	else if (ox + sizex > screenWidth)
+		ox = screenWidth - sizex;
 	if (oy + sizey > screenHeight)
-		oy -= oy + sizey - screenHeight;
+		oy = screenHeight - sizey;
 
 	gtk_widget_set_uposition(PWidget(id), ox, oy);
 #if 0
+
 	GtkAllocation alloc;
 	alloc.x = rc.left + ox;
 	alloc.y = rc.top + oy;
@@ -1725,7 +1841,8 @@ void Window::SetCursor(Cursor curs) {
 		break;
 	}
 
-	gdk_window_set_cursor(PWidget(id)->window, gdkCurs);
+	if (PWidget(id)->window)
+		gdk_window_set_cursor(PWidget(id)->window, gdkCurs);
 	gdk_cursor_destroy(gdkCurs);
 }
 
@@ -1801,9 +1918,10 @@ public:
 		}
 	}
 	virtual void SetFont(Font &font);
-	virtual void Create(Window &parent, int ctrlID, int lineHeight_, bool unicodeMode_);
+	virtual void Create(Window &parent, int ctrlID, Point location_, int lineHeight_, bool unicodeMode_);
 	virtual void SetAverageCharWidth(int width);
 	virtual void SetVisibleRows(int rows);
+	virtual int GetVisibleRows() const;
 	virtual PRectangle GetDesiredRect();
 	virtual int CaretFromEdge();
 	virtual void Clear();
@@ -1819,6 +1937,7 @@ public:
 		doubleClickAction = action;
 		doubleClickActionData = data;
 	}
+	virtual void SetList(const char* list, char separator, char typesep);
 };
 
 ListBox *ListBox::Allocate() {
@@ -1827,6 +1946,11 @@ ListBox *ListBox::Allocate() {
 }
 
 #if GTK_MAJOR_VERSION < 2
+static void UnselectionAC(GtkWidget *, gint, gint,
+                        GdkEventButton *, gpointer p) {
+	int *pi = reinterpret_cast<int *>(p);
+	*pi = -1;
+}
 static void SelectionAC(GtkWidget *, gint row, gint,
                         GdkEventButton *, gpointer p) {
 	int *pi = reinterpret_cast<int *>(p);
@@ -1844,7 +1968,7 @@ static gboolean ButtonPress(GtkWidget *, GdkEventButton* ev, gpointer p) {
 	return FALSE;
 }
 
-void ListBoxX::Create(Window &, int, int, bool) {
+void ListBoxX::Create(Window &, int, Point, int, bool) {
 	id = gtk_window_new(GTK_WINDOW_POPUP);
 
 	GtkWidget *frame = gtk_frame_new(NULL);
@@ -1866,6 +1990,8 @@ void ListBoxX::Create(Window &, int, int, bool) {
 	gtk_container_add(GTK_CONTAINER(PWidget(scroller)), PWidget(list));
 	gtk_clist_set_column_auto_resize(GTK_CLIST(PWidget(list)), 0, TRUE);
 	gtk_clist_set_selection_mode(GTK_CLIST(PWidget(list)), GTK_SELECTION_BROWSE);
+	gtk_signal_connect(GTK_OBJECT(PWidget(list)), "unselect_row",
+	                   GTK_SIGNAL_FUNC(UnselectionAC), &current);
 	gtk_signal_connect(GTK_OBJECT(PWidget(list)), "select_row",
 	                   GTK_SIGNAL_FUNC(SelectionAC), &current);
 	gtk_signal_connect(GTK_OBJECT(PWidget(list)), "button_press_event",
@@ -1902,9 +2028,6 @@ void ListBoxX::Create(Window &, int, int, bool) {
 	gtk_container_add(GTK_CONTAINER(PWidget(scroller)), PWidget(list));
 	gtk_widget_show(PWidget(list));
 
-	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store),
-										  TEXT_COLUMN, GTK_SORT_ASCENDING);
-
 	gtk_signal_connect(GTK_OBJECT(PWidget(list)), "button_press_event",
 	                   GTK_SIGNAL_FUNC(ButtonPress), this);
 #endif
@@ -1939,6 +2062,10 @@ void ListBoxX::SetVisibleRows(int rows) {
 	desiredVisibleRows = rows;
 }
 
+int ListBoxX::GetVisibleRows() const {
+	return desiredVisibleRows;
+}
+
 PRectangle ListBoxX::GetDesiredRect() {
 	// Before any size allocated pretend its 100 wide so not scrolled
 	PRectangle rc(0, 0, 100, 100);
@@ -2103,16 +2230,23 @@ int ListBoxX::Length() {
 
 void ListBoxX::Select(int n) {
 #if GTK_MAJOR_VERSION < 2
-	gtk_clist_select_row(GTK_CLIST(list), n, 0);
-	gtk_clist_moveto(GTK_CLIST(list), n, 0, 0.5, 0.5);
+	if (n == -1) {
+		gtk_clist_unselect_row(GTK_CLIST(list), current, 0);
+	} else {
+		gtk_clist_select_row(GTK_CLIST(list), n, 0);
+		gtk_clist_moveto(GTK_CLIST(list), n, 0, 0.5, 0.5);
+	}
 #else
-	if (n < 0)
-		return;
-
 	GtkTreeIter iter;
 	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
 	GtkTreeSelection *selection =
 		gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
+
+	if (n < 0) {
+		gtk_tree_selection_unselect_all(selection);
+		return;
+	}
+
 	bool valid = gtk_tree_model_iter_nth_child(model, &iter, NULL, n);
 	if (valid) {
 		gtk_tree_selection_select_iter(selection, &iter);
@@ -2147,6 +2281,8 @@ void ListBoxX::Select(int n) {
 
 		// Set it.
 		gtk_adjustment_set_value(adj, value);
+	} else {
+		gtk_tree_selection_unselect_all(selection);
 	}
 #endif
 }
@@ -2166,7 +2302,7 @@ int ListBoxX::GetSelection() {
 		if (indices)
 			return indices[0];
 	}
-	return 0;
+	return -1;
 #endif
 }
 
@@ -2196,7 +2332,7 @@ int ListBoxX::Find(const char *prefix) {
 		i++;
 	}
 #endif
-	return - 1;
+	return -1;
 }
 
 void ListBoxX::GetValue(int n, char *value, int len) {
@@ -2274,6 +2410,36 @@ void ListBoxX::ClearRegisteredImages() {
 	xset.Clear();
 }
 
+void ListBoxX::SetList(const char* list, char separator, char typesep) {
+	Clear();
+	int count = strlen(list) + 1;
+	char *words = new char[count];
+	if (words) {
+		memcpy(words, list, count);
+		char *startword = words;
+		char *numword = NULL;
+		int i = 0;
+		for (; words[i]; i++) {
+			if (words[i] == separator) {
+				words[i] = '\0';
+				if (numword)
+					*numword = '\0';
+				Append(startword, numword?atoi(numword + 1):-1);
+				startword = words + i + 1;
+				numword = NULL;
+			} else if (words[i] == typesep) {
+				numword = words + i;
+			}
+		}
+		if (startword) {
+			if (numword)
+				*numword = '\0';
+			Append(startword, numword?atoi(numword + 1):-1);
+		}
+		delete []words;
+	}
+}
+
 Menu::Menu() : id(0) {}
 
 void Menu::CreatePopUp() {
@@ -2301,7 +2467,12 @@ void Menu::Show(Point pt, Window &) {
 	if ((pt.y + requisition.height) > screenHeight) {
 		pt.y = screenHeight - requisition.height;
 	}
-	gtk_item_factory_popup(factory, pt.x - 4, pt.y, 3, 0);
+#if GTK_MAJOR_VERSION >= 2
+	gtk_item_factory_popup(factory, pt.x - 4, pt.y - 4, 3,
+		gtk_get_current_event_time());
+#else
+	gtk_item_factory_popup(factory, pt.x - 4, pt.y - 4, 3, 0);
+#endif
 }
 
 ElapsedTime::ElapsedTime() {
@@ -2433,10 +2604,8 @@ int Platform::DBCSCharLength(int codePage, const char *s) {
 		// Experimental and disabled code - change 999932 to 932 above to
 		// enable locale avoiding but expensive character length determination.
 		// Avoid locale with explicit use of iconv
-		iconv_t iconvhMeasure =
-			iconv_open("UCS-2", CharacterSetID(SC_CHARSET_SHIFTJIS));
-		size_t lenChar = MultiByteLenFromIconv(iconvhMeasure, s, strlen(s));
-		iconv_close(iconvhMeasure);
+		Converter convMeasure("UCS-2", CharacterSetID(SC_CHARSET_SHIFTJIS));
+		size_t lenChar = MultiByteLenFromIconv(convMeasure, s, strlen(s));
 		return lenChar;
 	} else {
 		int bytes = mblen(s, MB_CUR_MAX);
