@@ -48,7 +48,7 @@
 *   DATA DECLARATIONS
 */
 
-enum { NumTokens = 3 };
+enum { NumTokens = 8 };
 
 typedef enum eException {
     ExceptionNone, ExceptionEOF, ExceptionFormattingError,
@@ -79,7 +79,8 @@ typedef enum eKeywordId {
     KEYWORD_TRY, KEYWORD_TYPEDEF, KEYWORD_TYPENAME,
     KEYWORD_UNION, KEYWORD_UNSIGNED, KEYWORD_USING,
     KEYWORD_VIRTUAL, KEYWORD_VOID, KEYWORD_VOLATILE,
-    KEYWORD_WCHAR_T
+    KEYWORD_WCHAR_T, KEYWORD_AUTO, KEYWORD_RESTRICT, KEYWORD_SIZE_T,
+    KEYWORD_BOOL
 } keywordId;
 
 /*  Used to determine whether keyword is valid for the current language and
@@ -106,6 +107,7 @@ typedef enum eTokenType {
     TOKEN_PAREN_NAME,	/* a single name in parentheses */
     TOKEN_SEMICOLON,	/* the semicolon character */
     TOKEN_SPEC,		/* a storage class specifier, qualifier, type, etc. */
+    TOKEN_STAR,   /* pointer detection */
     TOKEN_COUNT
 } tokenType;
 
@@ -156,8 +158,11 @@ typedef struct sTokenInfo {
     tokenType     type;
     keywordId     keyword;
     vString*      name;		/* the name of the token */
+    int           pointerOrder; /* 0=no pointer, 1=pointer, 2=pointer
+				   to pointer, etc */
     unsigned long lineNumber;	/* line number of tag */
     fpos_t        filePosition;	/* file position of line containing name */
+	int			  bufferPosition;	/* buffer position of line containing name */
 } tokenInfo;
 
 
@@ -178,10 +183,11 @@ typedef struct sStatementInfo {
     boolean	haveQualifyingName;  /* do we have a name we are considering? */
     boolean	gotParenName;	/* was a name inside parentheses parsed yet? */
     boolean	gotArgs;	/* was a list of parameters parsed yet? */
-    boolean	isPointer;	/* is 'name' a pointer? */
+    int         pointerOrder;   /* 0=no pointer, 1=pointer, 2=pointer
+				   to pointer, etc */
     impType	implementation;	/* abstract or concrete implementation? */
     unsigned int tokenIndex;	/* currently active token */
-    tokenInfo*	token [(int) NumTokens];
+    tokenInfo*	token [((int) NumTokens)];
     tokenInfo*	context;	/* accumulated scope of current statement */
     tokenInfo*	blockName;	/* name of current block */
     memberInfo	member;		/* information regarding parent class/struct */
@@ -211,12 +217,12 @@ typedef enum eTagType {
     TAG_UNION,			/* union name */
     TAG_VARIABLE,		/* variable definition */
     TAG_EXTERN_VAR,		/* external variable declaration */
-	TAG_MACRO,			/* #define s */
+    TAG_MACRO,			/* #define s */
     TAG_COUNT			/* must be last */
 } tagType;
 
 typedef struct sParenInfo {
-    boolean isPointer;
+    int pointerOrder;
     boolean isParamList;
     boolean isKnrParamList;
     boolean isNameCandidate;
@@ -330,13 +336,18 @@ static const keywordDesc KeywordTable [] = {
     { "virtual",	KEYWORD_VIRTUAL,	{ 0, 1, 0 } },
     { "void",		KEYWORD_VOID,		{ 1, 1, 1 } },
     { "volatile",	KEYWORD_VOLATILE,	{ 1, 1, 1 } },
-    { "wchar_t",	KEYWORD_WCHAR_T,	{ 1, 1, 0 } }
+    { "wchar_t",	KEYWORD_WCHAR_T,	{ 1, 1, 0 } },
+    { "auto",		KEYWORD_AUTO,		{ 1, 1, 0 } },
+    { "restrict",	KEYWORD_RESTRICT,	{ 1, 0, 0 } },
+    { "size_t",	KEYWORD_SIZE_T,		{ 1, 1, 0 } },
+    { "bool",		KEYWORD_BOOL,		{ 0, 1, 0 } }
 };
 
 /*
 *   FUNCTION PROTOTYPES
 */
 static void createTags (const unsigned int nestLevel, statementInfo *const parent);
+static void copyToken (tokenInfo *const dest, const tokenInfo *const src);
 
 /*
 *   FUNCTION DEFINITIONS
@@ -346,7 +357,7 @@ static void createTags (const unsigned int nestLevel, statementInfo *const paren
 #if DEBUG_C
 static char *tokenTypeName[] = {
     "none", "args", "'}'", "'{'", "','", "'::'", "keyword", "name",
-    "package", "paren-name", "';'",	"spec", "count"
+    "package", "paren-name", "';'",	"spec", "*","count"
 };
 
 static char *tagScopeNames[] = {
@@ -401,15 +412,6 @@ void printStatement(const statementInfo *const statement)
 }
 #endif
 
-/* Checks if the token is a base data type (int/log/float/void/etc) */
-static boolean isBaseDataType(const char *name)
-{
-	return (boolean) ((0 == strcmp(name, "int")) || (0 == strcmp(name, "char")) ||
-	  (0 == strcmp(name, "long")) || (0 == strcmp(name, "float")) ||
-	  (0 == strcmp(name, "double")) || (0 == strcmp(name, "short")) ||
-	  (0 == strcmp(name, "void")) || (0 == strcmp(name, "bool")));
-}
-
 extern boolean includingDefineTags (void)
 {
     return CKinds [CK_DEFINE].enabled;
@@ -423,17 +425,26 @@ static void initToken (tokenInfo* const token)
 {
     token->type		= TOKEN_NONE;
     token->keyword	= KEYWORD_NONE;
+    token->pointerOrder = 0;
     token->lineNumber	= getSourceLineNumber ();
-    token->filePosition	= getInputFilePosition ();
+    
+	if (useFile())
+		token->filePosition	= getInputFilePosition ();
+	else
+		token->bufferPosition = getInputBufferPosition ();
     vStringClear (token->name);
 }
 
 static void advanceToken (statementInfo* const st)
 {
     if (st->tokenIndex >= (unsigned int) NumTokens - 1)
-	st->tokenIndex = 0;
+    {
+      st->tokenIndex = 0;
+    }
     else
-	++st->tokenIndex;
+    {
+      ++st->tokenIndex;
+    }
     initToken (st->token [st->tokenIndex]);
 }
 
@@ -443,6 +454,7 @@ static tokenInfo *prevToken (const statementInfo *const st, unsigned int n)
     unsigned int num = (unsigned int) NumTokens;
     Assert (n < num);
     tokenIndex = (st->tokenIndex + num - n) % num;
+
     return st->token [tokenIndex];
 }
 
@@ -457,9 +469,13 @@ static void setToken (statementInfo *const st, const tokenType type)
 static void retardToken (statementInfo *const st)
 {
     if (st->tokenIndex == 0)
-	st->tokenIndex = (unsigned int) NumTokens - 1;
+    {
+      st->tokenIndex = (unsigned int) NumTokens - 1;
+    }
     else
-	--st->tokenIndex;
+    {
+      --st->tokenIndex;
+    }
     setToken (st, TOKEN_NONE);
 }
 
@@ -475,8 +491,8 @@ static void deleteToken (tokenInfo *const token)
 {
     if (token != NULL)
     {
-	vStringDelete (token->name);
-	eFree (token);
+      vStringDelete (token->name);
+      eFree (token);
     }
 }
 
@@ -512,7 +528,7 @@ static const char *tokenString (const tokenType type)
 {
     static const char *const names [] = {
 	"none", "args", "}", "{", "comma", "double colon", "keyword", "name",
-	"package", "paren-name", "semicolon", "specifier"
+	"package", "paren-name", "semicolon", "specifier", "*","count"
     };
     Assert (sizeof (names) / sizeof (names [0]) == TOKEN_COUNT);
     Assert ((int) type < TOKEN_COUNT);
@@ -599,6 +615,46 @@ static void __unused__ ps (statementInfo *const st)
 *   Statement management
 */
 
+static boolean isDataTypeKeyword (const tokenInfo *const token)
+{
+    switch (token->keyword)
+    {
+      case KEYWORD_BOOLEAN:
+      case KEYWORD_BYTE:
+      case KEYWORD_CHAR:
+      case KEYWORD_DOUBLE:
+      case KEYWORD_FLOAT:
+      case KEYWORD_INT:
+      case KEYWORD_LONG:
+      case KEYWORD_SHORT:
+      case KEYWORD_VOID:
+      case KEYWORD_WCHAR_T:
+      case KEYWORD_SIZE_T:
+      case KEYWORD_BOOL:
+         return TRUE;
+      default: return FALSE;
+    }
+    return FALSE;
+}
+
+static boolean isVariableKeyword (const tokenInfo *const token)
+{
+    switch (token->keyword)
+    {
+      case KEYWORD_CONST:
+      case KEYWORD_EXTERN:
+      case KEYWORD_REGISTER:
+      case KEYWORD_STATIC:
+      case KEYWORD_AUTO:
+      case KEYWORD_RESTRICT:
+      case KEYWORD_VIRTUAL:
+      case KEYWORD_SIGNED:
+      case KEYWORD_UNSIGNED:
+	  return TRUE;
+      default: return FALSE;
+    }
+}
+
 static boolean isContextualKeyword (const tokenInfo *const token)
 {
     boolean result;
@@ -690,17 +746,19 @@ static void reinitStatement (statementInfo *const st, const boolean partial)
 	    st->declaration = DECL_NONE;
     }
     st->gotParenName	= FALSE;
-    st->isPointer	= FALSE;
+    st->pointerOrder    = 0;
     st->implementation	= IMP_DEFAULT;
     st->gotArgs		= FALSE;
     st->gotName		= FALSE;
     st->haveQualifyingName = FALSE;
+    st->argEndPosition = 0;
+		
     st->tokenIndex	= 0;
-	st->argEndPosition = 0;
-
     for (i = 0  ;  i < (unsigned int) NumTokens  ;  ++i)
-	initToken (st->token [i]);
-
+    {
+      initToken (st->token [i]);
+    }
+		
     initToken (st->context);
     initToken (st->blockName);
     vStringClear (st->parentClasses);
@@ -708,18 +766,38 @@ static void reinitStatement (statementInfo *const st, const boolean partial)
     /*  Init member info.
      */
     if (! partial)
-	st->member.access = st->member.accessDefault;
+	   st->member.access = st->member.accessDefault;
 	
 	/* Init first token */
 	if (!partial)
 	initToken(st->firstToken);
 }
 
+static void reinitStatementWithToken (statementInfo *const st,
+                                tokenInfo *token, const boolean partial)
+{
+    tokenInfo *const save = newToken ();
+    /* given token can be part of reinit statementInfo */
+    copyToken (save, token);
+    reinitStatement (st, partial);
+    token = activeToken (st);
+    copyToken (token, save);
+    deleteToken (save);
+    ++st->tokenIndex;	/* this is quite save becouse current tokenIndex = 0 */
+}
+
 static void initStatement (statementInfo *const st, statementInfo *const parent)
 {
     st->parent = parent;
     initMemberInfo (st);
     reinitStatement (st, FALSE);
+    if(parent)
+    {
+      const tokenInfo *const src = activeToken (parent);
+      tokenInfo *const dst = activeToken (st);
+      copyToken (dst, src);
+      st->tokenIndex++;
+    }
 }
 
 /*
@@ -850,36 +928,40 @@ static void addOtherFields (tagEntryInfo* const tag, const tagType type,
 	case TAG_TYPEDEF:
 	case TAG_UNION:
 	    if (isMember (st) &&
-		! (type == TAG_ENUMERATOR  &&  vStringLength (scope) == 0))
-	    {
-		if (isType (st->context, TOKEN_NAME))
-		    tag->extensionFields.scope [0] = tagName (TAG_CLASS);
-		else
-		    tag->extensionFields.scope [0] =
-			tagName (declToTagType (parentDecl (st)));
-		tag->extensionFields.scope [1] = vStringValue (scope);
+		! (type == TAG_ENUMERATOR  &&  vStringLength (scope) == 0))  {
+			if (isType (st->context, TOKEN_NAME))
+		    	tag->extensionFields.scope [0] = tagName (TAG_CLASS);
+			else
+		    	tag->extensionFields.scope [0] =
+					tagName (declToTagType (parentDecl (st)));
+			tag->extensionFields.scope [1] = vStringValue (scope);
 	    }
 	    if ((type == TAG_CLASS  ||  type == TAG_INTERFACE  ||
-		 type == TAG_STRUCT) && vStringLength (st->parentClasses) > 0)
-	    {
+		 type == TAG_STRUCT) && vStringLength (st->parentClasses) > 0) {
 
-		tag->extensionFields.inheritance =
-			vStringValue (st->parentClasses);
+			tag->extensionFields.inheritance =
+				vStringValue (st->parentClasses);
 	    }
 	    if (st->implementation != IMP_DEFAULT &&
-		(isLanguage (Lang_cpp) || isLanguage (Lang_java)))
-	    {
-		tag->extensionFields.implementation =
-			implementationString (st->implementation);
+				(isLanguage (Lang_cpp) || isLanguage (Lang_java)))  {
+			tag->extensionFields.implementation =
+				implementationString (st->implementation);
 	    }
-	    if (isMember (st))
-	    {
-		tag->extensionFields.access = accessField (st);
+	    if (isMember (st)) {
+			tag->extensionFields.access = accessField (st);
 	    }
 		if ((TRUE == st->gotArgs) && (TRUE == Option.extensionFields.argList) &&
-			((TAG_FUNCTION == type) || (TAG_METHOD == type) || (TAG_PROTOTYPE == type)))
-				tag->extensionFields.arglist = getArglistFromPos(
+			((TAG_FUNCTION == type) || (TAG_METHOD == type) || (TAG_PROTOTYPE == type))) {
+			
+			if (useFile()) {
+				tag->extensionFields.arglist = getArglistFromFilePos(
 				  tag->filePosition, tag->name);
+			}
+			else {
+				tag->extensionFields.arglist = getArglistFromBufferPos(
+				  tag->bufferPosition, tag->name);
+			}
+		}
 	    break;
     }
 	
@@ -888,8 +970,7 @@ static void addOtherFields (tagEntryInfo* const tag, const tagType type,
 		(TAG_VARIABLE == tag->type) || (TAG_METHOD == tag->type) ||
 		(TAG_PROTOTYPE == tag->type) || (TAG_FUNCTION == tag->type))
 	{
-		if (((TOKEN_NAME == st->firstToken->type) || ((TOKEN_KEYWORD == st->firstToken->type)
-			  && isBaseDataType(st->firstToken->name->buffer)))
+		if (((TOKEN_NAME == st->firstToken->type) || isDataTypeKeyword(st->firstToken))
 			  && (0 != strcmp(vStringValue(st->firstToken->name), tag->name)))
 				tag->extensionFields.varType = vStringValue(st->firstToken->name);
 	}
@@ -1006,12 +1087,18 @@ static void makeTag (const tokenInfo *const token,
 	initTagEntry (&e, vStringValue (token->name));
 
 	e.lineNumber	= token->lineNumber;
-	e.filePosition	= token->filePosition;
+	
+	if (useFile())
+		e.filePosition	= token->filePosition;
+	else
+		e.bufferPosition = token->bufferPosition;
+	
 	e.isFileScope	= isFileScope;
 	e.kindName	= tagName (type);
 	e.kind		= tagLetter (type);
 	e.type = type;
-
+	e.pointerOrder = token->pointerOrder;
+		
 	findScopeHierarchy (scope, st);
 	addOtherFields (&e, type, st, scope);
 
@@ -1157,8 +1244,9 @@ static int skipToNonWhite (void)
 {
     int c;
 
-    do
-	c = cppGetc ();
+    do {
+		c = cppGetc ();
+	}
     while (isspace (c));
 
     return c;
@@ -1415,8 +1503,12 @@ static void copyToken (tokenInfo *const dest, const tokenInfo *const src)
 {
     dest->type         = src->type;
     dest->keyword      = src->keyword;
-    dest->filePosition = src->filePosition;
+    if (useFile())
+		dest->filePosition = src->filePosition;
+	else
+		dest->bufferPosition = src->bufferPosition;
     dest->lineNumber   = src->lineNumber;
+    dest->pointerOrder = src->pointerOrder;
     vStringCopy (dest->name, src->name);
 }
 
@@ -1429,7 +1521,7 @@ static void setAccess (statementInfo *const st, const accessType access)
 	    int c = skipToNonWhite ();
 
 	    if (c == ':')
-		reinitStatement (st, FALSE);
+        reinitStatementWithToken (st, prevToken (st, 1), FALSE);
 	    else
 		cppUngetc (c);
 
@@ -1510,6 +1602,8 @@ static void processToken (tokenInfo *const token, statementInfo *const st)
 	case KEYWORD_CHAR:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_CLASS:	st->declaration = DECL_CLASS;		break;
 	case KEYWORD_CONST:	st->declaration = DECL_BASE;		break;
+	case KEYWORD_AUTO:	st->declaration = DECL_BASE;		break;
+	case KEYWORD_RESTRICT: st->declaration = DECL_BASE;		break;
 	case KEYWORD_DOUBLE:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_ENUM:	st->declaration = DECL_ENUM;		break;
 	case KEYWORD_EXTENDS:	readParents (st, '.');
@@ -1520,6 +1614,9 @@ static void processToken (tokenInfo *const token, statementInfo *const st)
 				setToken (st, TOKEN_NONE);		break;
 	case KEYWORD_IMPORT:	st->declaration = DECL_IGNORE;		break;
 	case KEYWORD_INT:	st->declaration = DECL_BASE;		break;
+	case KEYWORD_BOOL:	st->declaration = DECL_BASE;		break;
+	case KEYWORD_WCHAR_T:	st->declaration = DECL_BASE;		break;
+	case KEYWORD_SIZE_T:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_INTERFACE: st->declaration = DECL_INTERFACE;	break;
 	case KEYWORD_LONG:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_NAMESPACE: st->declaration = DECL_NAMESPACE;	break;
@@ -1790,11 +1887,13 @@ static int parseParens (statementInfo *const st, parenInfo *const info)
     do
     {
 	int c = skipToNonWhite ();
+
 	switch (c)
 	{
-	    case '&':
-	    case '*':
-		info->isPointer = TRUE;
+	    case '&':	
+	    case '*':	
+		/* DEBUG_PRINT("parseParens, po++\n"); */
+		info->pointerOrder++;
 		info->isKnrParamList = FALSE;
 		if (identifierCount == 0)
 		    info->isParamList = FALSE;
@@ -1908,8 +2007,12 @@ static int parseParens (statementInfo *const st, parenInfo *const info)
 	skipToMatch ("()");
 	--depth;
     }
-	if (st->argEndPosition == 0)
-		st->argEndPosition = ftell(File.fp);
+	if (st->argEndPosition == 0) {
+		if (useFile())
+			st->argEndPosition = ftell(File.fp);
+		else
+			st->argEndPosition = File.fpBufferPosition;
+	}
 
     if (! info->isNameCandidate)
 	initToken (token);
@@ -1919,7 +2022,7 @@ static int parseParens (statementInfo *const st, parenInfo *const info)
 
 static void initParenInfo (parenInfo *const info)
 {
-    info->isPointer		= FALSE;
+    info->pointerOrder          = 0;
     info->isParamList		= TRUE;
     info->isKnrParamList	= TRUE;
     info->isNameCandidate	= TRUE;
@@ -1934,39 +2037,43 @@ static void analyzeParens (statementInfo *const st)
 
     if (! isType (prev, TOKEN_NONE))    /* in case of ignored enclosing macros */
     {
-	tokenInfo *const token = activeToken (st);
-	parenInfo info;
-	int c;
+		tokenInfo *const token = activeToken (st);
+		parenInfo info;
+		int c;
 
-	initParenInfo (&info);
-	parseParens (st, &info);
-	c = skipToNonWhite ();
-	cppUngetc (c);
-	if (info.invalidContents)
-	    reinitStatement (st, FALSE);
-	else if (info.isNameCandidate  &&  isType (token, TOKEN_PAREN_NAME)  &&
-		 ! st->gotParenName  &&
-		 (! info.isParamList || ! st->haveQualifyingName  ||
-		  c == '('  ||
-		  (c == '='  &&  st->implementation != IMP_VIRTUAL) ||
-		  (st->declaration == DECL_NONE  &&  isOneOf (c, ",;"))))
-	{
-	    token->type = TOKEN_NAME;
-	    processName (st);
-	    st->gotParenName = TRUE;
-	    if (! (c == '('  &&  info.nestedArgs))
-		st->isPointer = info.isPointer;
+		initParenInfo (&info);
+		parseParens (st, &info);
+	
+		c = skipToNonWhite ();
+
+		cppUngetc (c);
+		if (info.invalidContents) {
+	    	reinitStatement (st, FALSE);
+		}
+		else if (info.isNameCandidate  &&  isType (token, TOKEN_PAREN_NAME)  &&
+			 ! st->gotParenName  &&
+		 	(! info.isParamList || ! st->haveQualifyingName  ||
+		  	c == '('  ||
+		  	(c == '='  &&  st->implementation != IMP_VIRTUAL) ||
+		  	(st->declaration == DECL_NONE  &&  isOneOf (c, ",;"))))
+		{
+	    	token->type = TOKEN_NAME;
+	    	processName (st);
+	    	st->gotParenName = TRUE;
+	    	if (! (c == '('  &&  info.nestedArgs))
+				st->pointerOrder = info.pointerOrder;
+		}
+		else if (! st->gotArgs  &&  info.isParamList)
+		{
+		    st->gotArgs = TRUE;
+		    setToken (st, TOKEN_ARGS);
+		    advanceToken (st);
+		    analyzePostParens (st, &info);
+		}
+		else {
+		    setToken (st, TOKEN_NONE);
+		}
 	}
-	else if (! st->gotArgs  &&  info.isParamList)
-	{
-	    st->gotArgs = TRUE;
-	    setToken (st, TOKEN_ARGS);
-	    advanceToken (st);
-	    analyzePostParens (st, &info);
-	}
-	else
-	    setToken (st, TOKEN_NONE);
-    }
 }
 
 /*
@@ -2107,42 +2214,41 @@ static void parseGeneralToken (statementInfo *const st, const int c)
  */
 static void nextToken (statementInfo *const st)
 {
+    int c;
     tokenInfo *token = activeToken (st);
     do
     {
-
-	int c = skipToNonWhite ();
-
-	switch (c)
-	{
-	    case EOF: longjmp (Exception, (int) ExceptionEOF);		break;
-	    case '(': analyzeParens (st); token = activeToken (st); break;
-	    case '*': st->haveQualifyingName = FALSE;			break;
-	    case ',': setToken (st, TOKEN_COMMA);			break;
-	    case ':': processColon (st);					break;
-	    case ';': setToken (st, TOKEN_SEMICOLON);			break;
-	    case '<': skipToMatch ("<>");				break;
-	    case '=': processInitializer (st);				break;
-	    case '[': skipToMatch ("[]");				break;
-	    case '{': setToken (st, TOKEN_BRACE_OPEN);			break;
-	    case '}': setToken (st, TOKEN_BRACE_CLOSE);			break;
-	    default:  parseGeneralToken (st, c);				break;
-	}
+		c = skipToNonWhite();
+		switch (c)
+		{
+		    case EOF: longjmp (Exception, (int) ExceptionEOF);		break;
+		    case '(': analyzeParens (st); token = activeToken (st);     break;
+	    	case '*': setToken (st, TOKEN_STAR);			break;
+		    case ',': setToken (st, TOKEN_COMMA);			break;
+		    case ':': processColon (st);				      break;
+		    case ';': setToken (st, TOKEN_SEMICOLON);			break;
+		    case '<': skipToMatch ("<>");				break;
+	    	case '=': processInitializer (st);				break;
+		    case '[': skipToMatch ("[]");				break;
+		    case '{': setToken (st, TOKEN_BRACE_OPEN);			break;
+	    	case '}': setToken (st, TOKEN_BRACE_CLOSE);			break;
+	    	default:  parseGeneralToken (st, c);				break;
+		}
     } while (isType (token, TOKEN_NONE));
 	
 	/* We want to know about non-keyword variable types */
 	if (TOKEN_NONE == st->firstToken->type)
 	{
-		if ((TOKEN_NAME == token->type) || ((TOKEN_KEYWORD == token->type)
-			&& (isBaseDataType(token->name->buffer))))
+	    if ((TOKEN_NAME == token->type) || isDataTypeKeyword(token)) {
 			copyToken(st->firstToken, token);
+		}
 	}
 }
 
 /*
 *   Scanning support functions
 */
-
+static unsigned int contextual_fake_count = 0;
 static statementInfo *CurrentStatement = NULL;
 
 static statementInfo *newStatement (statementInfo *const parent)
@@ -2156,7 +2262,7 @@ static statementInfo *newStatement (statementInfo *const parent)
     st->context = newToken ();
     st->blockName = newToken ();
     st->parentClasses = vStringNew ();
-	st->firstToken = newToken();
+    st->firstToken = newToken();
 
     initStatement (st, parent);
     CurrentStatement = st;
@@ -2177,7 +2283,7 @@ static void deleteStatement (void)
     deleteToken (st->blockName);           st->blockName = NULL;
     deleteToken (st->context);             st->context = NULL;
     vStringDelete (st->parentClasses);     st->parentClasses = NULL;
-	deleteToken(st->firstToken);
+    deleteToken(st->firstToken);
     eFree (st);
     CurrentStatement = parent;
 }
@@ -2196,8 +2302,7 @@ static boolean isStatementEnd (const statementInfo *const st)
     if (isType (token, TOKEN_SEMICOLON))
 	isEnd = TRUE;
     else if (isType (token, TOKEN_BRACE_CLOSE))
-	isEnd = (boolean) (isLanguage (Lang_java) ||
-			  ! isContextualStatement (st));
+	isEnd = (boolean) (isLanguage (Lang_java) || !isContextualStatement (st));
     else
 	isEnd = FALSE;
 
@@ -2207,13 +2312,14 @@ static boolean isStatementEnd (const statementInfo *const st)
 static void checkStatementEnd (statementInfo *const st)
 {
     const tokenInfo *const token = activeToken (st);
-
-    if (isType (token, TOKEN_COMMA))
-	reinitStatement (st, TRUE);
-    else if (isStatementEnd (st))
-    {
+    boolean comma = isType (token, TOKEN_COMMA);
+	
+    if (comma || isStatementEnd (st))
+    {		
+      reinitStatementWithToken (st, activeToken (st), comma);
+        
+	
 	DebugStatement ( if (debug (DEBUG_PARSE)) printf ("<ES>"); )
-	reinitStatement (st, FALSE);
 	cppEndStatement ();
     }
     else
@@ -2244,6 +2350,28 @@ static void nest (statementInfo *const st, const unsigned int nestLevel)
     setToken (st, TOKEN_BRACE_CLOSE);
 }
 
+static int getTokenPointerOrder (statementInfo *const st, int prev)
+{
+    const tokenInfo *prev_tmp = prevToken (st, prev);
+    int pointer_order = 0;
+
+    /* ... , *const ptr; */
+    if (isVariableKeyword ( prev_tmp ))
+    {
+	prev++;
+	prev_tmp = prevToken (st, prev);
+    }
+
+    while (isType ( prev_tmp, TOKEN_STAR ))
+    {
+	pointer_order++;
+	prev++;
+	prev_tmp = prevToken (st, prev);
+    }
+
+    return pointer_order;
+}
+
 static void tagCheck (statementInfo *const st)
 {
     const tokenInfo *const token = activeToken (st);
@@ -2270,17 +2398,47 @@ static void tagCheck (statementInfo *const st)
 		    st->declaration = DECL_FUNCTION;
 		    if (isType (prev2, TOKEN_NAME))
 			copyToken (st->blockName, prev2);
+
+		    if (!isLanguage (Lang_java))
+		    {
+			((tokenInfo *)prev2)->pointerOrder = getTokenPointerOrder (st, 3);
+		    }
 		    qualifyFunctionTag (st, prev2);
 		}
 	    }
 	    else if (isContextualStatement (st))
 	    {
-		if (isType (prev, TOKEN_NAME))
-		    copyToken (st->blockName, prev);
-		qualifyBlockTag (st, prev);
+		tokenInfo *name_token = (tokenInfo *)prev;
+		if (isType (name_token, TOKEN_NAME))
+		{
+		    copyToken (st->blockName, name_token);
+		}
+		else
+		{
+		    tokenInfo *contextual_token = (tokenInfo *)prev;
+		    if(isContextualKeyword (contextual_token))
+		    {
+			char buffer[64];
+			sprintf(buffer, "_fake_%d", contextual_fake_count++);
+
+			name_token = newToken ();
+			copyToken (name_token, contextual_token);
+			vStringCatS(name_token->name, buffer);
+
+			name_token->type = TOKEN_NAME;
+			name_token->keyword	= KEYWORD_NONE;
+
+			advanceToken (st);
+			contextual_token = activeToken (st);
+			copyToken (contextual_token, token);
+			copyToken ((tokenInfo *const)token, name_token);
+			copyToken (st->blockName, name_token);
+			copyToken (st->firstToken, name_token);
+		    }   
+		}
+		qualifyBlockTag (st, name_token);
 	    }
 	    break;
-
 	case TOKEN_SEMICOLON:
 	case TOKEN_COMMA:
 	    if (insideEnumBody (st))
@@ -2290,14 +2448,21 @@ static void tagCheck (statementInfo *const st)
 		if (isContextualKeyword (prev2))
 		    st->scope = SCOPE_EXTERN;
 		else
-			qualifyVariableTag (st, prev);
+		{
+		    if (!isLanguage (Lang_java))
+		    {
+			((tokenInfo *)prev)->pointerOrder = getTokenPointerOrder (st, 2);
+		    }
+		    qualifyVariableTag (st, prev);
+		}
 	    }
 	    else if (isType (prev, TOKEN_ARGS)  &&  isType (prev2, TOKEN_NAME))
 	    {
-		if (st->isPointer)
-		    qualifyVariableTag (st, prev2);
-		else
-		    qualifyFunctionDeclTag (st, prev2);
+		if (!isLanguage (Lang_java))
+		{
+		    ((tokenInfo *)prev2)->pointerOrder = getTokenPointerOrder (st, 3);
+		}
+		qualifyFunctionDeclTag(st, prev2);
 	    }
 	    break;
 
@@ -2314,35 +2479,33 @@ static void createTags (const unsigned int nestLevel,
     statementInfo *const st = newStatement (parent);
 
     DebugStatement ( if (nestLevel > 0) debugParseNest (TRUE, nestLevel); )
-    while (TRUE)
-    {
-	tokenInfo *token;
+    while (TRUE) {
+		tokenInfo *token;
 
-	nextToken (st);
-	token = activeToken (st);
-	if (isType (token, TOKEN_BRACE_CLOSE))
-	{
-	    if (nestLevel > 0)
-		break;
-	    else
-	    {
-		verbose ("%s: unexpected closing brace at line %lu\n",
-			getInputFileName (), getInputLineNumber ());
-		longjmp (Exception, (int) ExceptionBraceFormattingError);
-	    }
-	}
-	else if (isType (token, TOKEN_DOUBLE_COLON))
-	{
-	    addContext (st, prevToken (st, 1));
-	    advanceToken (st);
-	}
-	else
-	{
-	    tagCheck (st);
-	    if (isType (token, TOKEN_BRACE_OPEN))
-		nest (st, nestLevel + 1);
-	    checkStatementEnd (st);
-	}
+		nextToken (st);
+
+		token = activeToken (st);
+
+		if (isType (token, TOKEN_BRACE_CLOSE)) {
+	    	if (nestLevel > 0) {
+				break;
+			}
+	    	else {
+				verbose ("%s: unexpected closing brace at line %lu\n",
+					getInputFileName (), getInputLineNumber ());
+				longjmp (Exception, (int) ExceptionBraceFormattingError);
+	    	}
+		}
+		else if (isType (token, TOKEN_DOUBLE_COLON)) {
+	    	addContext (st, prevToken (st, 1));
+	    	advanceToken (st);
+		}
+		else {
+	    	tagCheck (st);/* this can add new token */
+	    	if (isType (activeToken (st), TOKEN_BRACE_OPEN))
+				nest (st, nestLevel + 1);
+	    	checkStatementEnd (st);
+		}
     }
     deleteStatement ();
     DebugStatement ( if (nestLevel > 0) debugParseNest (FALSE, nestLevel - 1); )
@@ -2358,17 +2521,17 @@ static boolean findCTags (const unsigned int passCount)
 
     exception = (exception_t) setjmp (Exception);
     retry = FALSE;
-    if (exception == ExceptionNone)
-	createTags (0, NULL);
-    else
-    {
-	deleteAllStatements ();
-	if (exception == ExceptionBraceFormattingError  &&  passCount == 1)
-	{
-	    retry = TRUE;
-	   verbose ("%s: retrying file with fallback brace matching algorithm\n",
-		    getInputFileName ());
+
+    if (exception == ExceptionNone) {
+	    createTags (0, NULL);
 	}
+    else {
+		deleteAllStatements ();
+		if (exception == ExceptionBraceFormattingError  &&  passCount == 1) {
+	    	retry = TRUE;
+	   		verbose ("%s: retrying file with fallback brace matching algorithm\n",
+		    	getInputFileName ());
+		}
     }
     cppTerminate ();
     return retry;
@@ -2388,18 +2551,21 @@ static void buildKeywordHash (const langType language, unsigned int idx)
 
 static void initializeCParser (const langType language)
 {
+    contextual_fake_count = 0;
     Lang_c = language;
     buildKeywordHash (language, 0);
 }
 
 static void initializeCppParser (const langType language)
 {
+    contextual_fake_count = 0;
     Lang_cpp = language;
     buildKeywordHash (language, 1);
 }
 
 static void initializeJavaParser (const langType language)
 {
+    contextual_fake_count = 0;
     Lang_java = language;
     buildKeywordHash (language, 2);
 }
