@@ -1,5 +1,4 @@
 /*
-*   $Id: parse.c,v 1.1 2001/11/22 06:58:33 naba Exp $
 *
 *   Copyright (c) 1996-2001, Darren Hiebert
 *
@@ -44,7 +43,7 @@ tagEntryFunction TagEntryFunction = NULL;
 extern void makeSimpleTag (const vString* const name,
 			   kindOption* const kinds, const int kind)
 {
-    if (kinds [kind].enabled  &&  name != NULL  &&  vStringLength (name) > 0)
+    if (name != NULL  &&  vStringLength (name) > 0)
     {
         tagEntryInfo e;
         initTagEntry (&e, vStringValue (name));
@@ -56,6 +55,26 @@ extern void makeSimpleTag (const vString* const name,
     }
 }
 
+extern void makeSimpleScopedTag (const vString* const name,
+				 kindOption* const kinds, const int kind,
+				 const char* scope, const char *scope2,
+				 const char *access)
+{
+    if (name != NULL  &&  vStringLength (name) > 0)
+    {
+        tagEntryInfo e;
+        initTagEntry (&e, vStringValue (name));
+
+        e.kindName = kinds [kind].name;
+        e.kind     = kinds [kind].letter;
+	e.extensionFields.scope[0] = scope;
+	e.extensionFields.scope[1] = scope2;
+	e.extensionFields.access = access;
+
+        makeTagEntry (&e);
+    }
+}
+
 /*
 *   parserDescription mapping management
 */
@@ -626,24 +645,24 @@ extern boolean parseFile (const char *const fileName)
     boolean tagFileResized = FALSE;
     langType language = Option.language;
     if (Option.language == LANG_AUTO)
-	language = getFileLanguage (fileName);
+		language = getFileLanguage (fileName);
     Assert (language != LANG_AUTO);
     if (language == LANG_IGNORE)
-	verbose ("ignoring %s (unknown language)\n", fileName);
+		verbose ("ignoring %s (unknown language)\n", fileName);
     else if (! LanguageTable [language]->enabled)
-	verbose ("ignoring %s (language disabled)\n", fileName);
+		verbose ("ignoring %s (language disabled)\n", fileName);
     else
     {
-	if (Option.filter)
-	    openTagFile ();
+		if (Option.filter)
+		    openTagFile ();
 
-	tagFileResized = createTagsWithFallback (fileName, language);
+		tagFileResized = createTagsWithFallback (fileName, language);
 
-	if (Option.filter)
-	    closeTagFile (tagFileResized);
-	addTotals (1, 0L, 0L);
+		if (Option.filter)
+	    	closeTagFile (tagFileResized);
+		addTotals (1, 0L, 0L);
 
-	return tagFileResized;
+		return tagFileResized;
     }
     return tagFileResized;
 }
